﻿;==========================================================================================================================
; Decode main XML data from the XML import file
;==========================================================================================================================

Procedure ImportRecipeMuncherFile_XML(*CurrentNode, CurrentSublevel.i)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------

  Protected NodeName.s, *ChildNode
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
  
    NodeName.s = GetXMLNodeName(*CurrentNode)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
  
    If NodeName.s = "RecipeItem"
      
      If Recipe\Title <> ""
        
        ImportRecipeFileSave(#Program)
        
        ClearRecipeStructure()
        
      EndIf
      
    ElseIf NodeName.s     = "Version"
      
      Recipe\Version       = RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Owner"
      
      Recipe\Owner         + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Title"
      
      Recipe\Title         = RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Categories"
      
      Recipe\Categories    + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Serves"
      
      Recipe\Serves        + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Source"
      
      Recipe\Source        + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Ingredients"
      
      Recipe\Ingredients   + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "Instructions"
      
      Recipe\Instructions  + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    ElseIf NodeName.s     = "PictureInfo"                                             ; 
      
      If ExamineXMLAttributes(*CurrentNode)                                            ; 
        
        While NextXMLAttribute(*CurrentNode)                                           ; 
          
          AttributeName.s = XMLAttributeName(*CurrentNode)                              ; 
          
          If AttributeName.s = "PictureType"                                           ;  Picture type
          
            FileNameNameType.s = XMLAttributeValue(*CurrentNode)                       ; 
          
          ElseIf AttributeName.s = "PictureData"                                      ;
            
            FileNameData.s = XMLAttributeValue(*CurrentNode)                           ; Picture data
            
          EndIf                                                                       ; 
          
        Wend                                                                          ; 
        
      EndIf                                                                           ; 

      If FilenameString.s = ""                                                        ; If the filename came back empty
        
        FilenameString.s = Recipe\Title                                               ; Add the recipe title as the filename
        
      EndIf 
      
      ;--------------------------------------------------------------------------------------------
      ; Kill illegal, non filesystem allowed characters from the generated picture filename
      ;--------------------------------------------------------------------------------------------
      
      For KillBadChars.i = 1 To Len(Program\IllegalCharacters)
        
        FilenameString.s  = ReplaceString(FilenameString.s, Mid(Program\IllegalCharacters, KillBadChars.i, 1), " ", #PB_String_NoCase)
        
      Next KillBadChars.i
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      Recipe\PictureName   + RecipeCRLFSpaceTrim(XmlToText(FileNameString.s))
      
      ;--------------------------------------------------------------------------------------------
      ; Check to see if this picture already exists on disk
      ;--------------------------------------------------------------------------------------------
      
      If FileSize(FilenameString.s) = -1 And FileSize(FilenameString.s) <> 0
        
        ;------------------------------------------------------------------------------------------
        ; Try to create the new output picture file
        ;------------------------------------------------------------------------------------------
        
        PictureFileHandle.i = CreateFile(#PB_Any, FilenameString.s)
        
        ;------------------------------------------------------------------------------------------
        ; Proceed if we got a disk file handle
        ;------------------------------------------------------------------------------------------
        
        If PictureFileHandle.i
          
          If Len(FileNameData.s)
            
            *Buffer = AllocateMemory(Len(FileNameData.s))
            
            If *Buffer
              
              CurrentXMLDataLineSize.i = Base64Decoder(@PictureFileXMLData, Len(FileNameData.s), *Buffer, MemorySize(*Buffer))
              
              If CurrentXMLDataLineSize.i
                
                If WriteData(PictureFileHandle.i, *Buffer, CurrentXMLDataLineSize.i) <> 0
                  
                  ; The write was okay, Str(CurrentXMLDataLineSize.i) bytes written"
                  
                Else
                  
                  ; Debug "There was an error writing the data out."
                  
                EndIf
                
              Else
                
                ; Debug "Picture is " + Str(CurrentXMLDataLineSize.i) + " bytes, nothing to do."
                
              EndIf
              
              FreeMemory(*Buffer)
              
            Else
              
              ; Debug "Could not allocate a memory buffer as the file it was for is empty."
              
            EndIf
            
          Else
            
            ; Debug "This line of data happens to be empty so we cannot allocate memory for it."
            
          EndIf
          
          ;--------------------------------------------------------------------------------------
          ; No recipe pictures found in the current file.
          ;--------------------------------------------------------------------------------------
          
        EndIf
        
        CloseFile(PictureFileHandle.i)
        
        FileNameData.s = ""
        
      Else
        
        ; File already exists at the location
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s     = "LastUpdate"
      
      Recipe\LastUpdate    + RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode)))
      
    EndIf
    
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    While *ChildNode <> 0
      
      ImportRecipeMuncherFile_XML(*ChildNode, CurrentSublevel + 1)
      
      *ChildNode = NextXMLNode(*ChildNode)
      
    Wend    
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
  
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure 


; IDE Options = PureBasic 5.10 (Windows - x64)
; CursorPosition = 220
; FirstLine = 174
; EnableXP