﻿;==============================================================================================================================================================================
; Try to open the system database and create missing tables
;==============================================================================================================================================================================

Procedure OpenSystemDatabase()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected.i FileHandle.i
  
  Protected.s DatabaseUpdate.s
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  FileHandle.i = OpenFile(#PB_Any, Program\DatabaseName)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If FileHandle.i
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFile(FileHandle.i)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DatabaseHandle = OpenDatabase(#PB_Any, Program\DatabaseName, #EmptyString, #EmptyString, #PB_Database_SQLite)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    If Program\DatabaseHandle <> #DatabaseOpenFail
      
      ;--------------------------------------------------------------------------------------------
      ; Turn on auto database vacuum
      ;--------------------------------------------------------------------------------------------
      
      If Not DatabaseUpdate(Program\DatabaseHandle, "PRAGMA auto_vacuum = on")                    ; 
        
        ;Debug "Could not setup auto database vacuum: " + DatabaseError()
        
      EndIf                                                                                       ; 
      
      ;--------------------------------------------------------------------------------------------
      ; Create the Recipes table
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Recipes("                                    ; 
      
      DatabaseUpdate.s + "Recipetitle TEXT, "                                                     ; Title
      
      DatabaseUpdate.s + "Numberofservings TEXT, "                                                ; YieldText (YieldNumber not used)
      
      DatabaseUpdate.s + "Recipeauthor TEXT, "                                                    ; 
      
      DatabaseUpdate.s + "Categories TEXT, "                                                      ; PrimaryIngredient
      
      DatabaseUpdate.s + "Subcategories TEXT, "                                                   ; AllCategoriesText
      
      DatabaseUpdate.s + "Preparationtime TEXT, "                                                 ; 
      
      DatabaseUpdate.s + "Cookingtime TEXT, "                                                     ; 
      
      DatabaseUpdate.s + "Difficulty TEXT, "                                                      ; 
      
      DatabaseUpdate.s + "Recipeversion TEXT, "                                                   ; 
      
      DatabaseUpdate.s + "Recipesource TEXT, "                                                    ; Source
      
      DatabaseUpdate.s + "Copyright TEXT, "                                                       ; 
      
      DatabaseUpdate.s + "Cuisine TEXT, "                                                         ; Cuisine
      
      DatabaseUpdate.s + "Reciperating TEXT, "                                                    ; 
      
      DatabaseUpdate.s + "Importedfrom TEXT, "                                                    ; 
      
      DatabaseUpdate.s + "Authorcomments TEXT, "                                                  ; 
      
      DatabaseUpdate.s + "Instructions TEXT, "                                                    ; Instructions
      
      DatabaseUpdate.s + "Nutritionaldata TEXT, "                                                 ; 
      
      DatabaseUpdate.s + "Othercomments TEXT, "                                                   ; Notes
      
      DatabaseUpdate.s + "Deleted TEXT, "                                                         ; 
      
      DatabaseUpdate.s + "Updated TEXT, "                                                         ; 
      
      DatabaseUpdate.s + "Favourite TEXT, "                                                       ; 
      
      DatabaseUpdate.s + "Locked TEXT, "                                                          ; 
      
      DatabaseUpdate.s + "Recordid INTEGER PRIMARY KEY AUTOINCREMENT, "                           ; RecipeID
      
      DatabaseUpdate.s + "UNIQUE (Recipetitle, Instructions) ON CONFLICT FAIL)"                   ; 
      
      If DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)                                 ; 
        
      Else                                                                                        ; 
        
        Debug "Could not create recipe table in database: " + DatabaseError()                     ; 
        
      EndIf                                                                                       ; 
      
      ;--------------------------------------------------------------------------------------------
      ; Create the index table
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE INDEX IF NOT EXISTS RecipeIndex ON Recipes(Recipetitle)"         ; 
      
      If DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)                                 ; 
        
      Else                                                                                        ; 
        
        Debug "Could not create index table in database: " + DatabaseError()                      ; 
        
      EndIf                                                                                       ; 
      
      ;--------------------------------------------------------------------------------------------
      ; Create the picture table
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Pictures("                                   ; 
      
      DatabaseUpdate.s + "Pictureid INTEGER PRIMARY KEY AUTOINCREMENT, "                          ; PictureID
      
      DatabaseUpdate.s + "Picture BLOB, "                                                         ; 
      
      DatabaseUpdate.s + "Description TEXT, "                                                     ; 
      
      DatabaseUpdate.s + "Filename TEXT, "                                                        ; 
      
      DatabaseUpdate.s + "Recordid INTEGER, "                                                     ; 
      
      DatabaseUpdate.s + "FOREIGN KEY(Recordid) REFERENCES Recipes(Recordid) ON DELETE CASCADE)"  ; 
      
      If Not DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)                             ; 
        
        Debug DatabaseUpdate.s  + "  --  "  + DatabaseError()                                     ; 
        
      EndIf                                                                                       ; 
      
      ;--------------------------------------------------------------------------------------------
      ; Create the ingredients table
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate.s = "CREATE TABLE IF NOT EXISTS Ingredients("                                ; 
      
      DatabaseUpdate.s + "Ingredientid INTEGER PRIMARY KEY AUTOINCREMENT, "                       ; IngredientLinesID
      
      DatabaseUpdate.s + "Unit TEXT, "                                                            ; TextQty
      
      DatabaseUpdate.s + "Measure TEXT, "                                                         ; Measure
      
      DatabaseUpdate.s + "Ingredient TEXT, "                                                      ; Name
      
      DatabaseUpdate.s + "Preparation TEXT, "                                                     ; PrepNotes
      
      DatabaseUpdate.s + "Lineorder TEXT, "                                                       ; LineOrder
      
      DatabaseUpdate.s + "Recordid INTEGER, "                                                     ; RecipeID
      
      DatabaseUpdate.s + "FOREIGN KEY(Recordid) REFERENCES Recipes(Recordid) ON DELETE CASCADE)"  ; 
      
      If DatabaseUpdate(Program\DatabaseHandle, DatabaseUpdate.s)                                 ; 
        
      Else                                                                                        ; 
        
        Debug "Could not create ingredients table: "  + DatabaseError()                           ; 
        
      EndIf                                                                                       ; 
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      Debug "There was a serious problem attempting to connect to " + Program\DatabaseName + " system database. Could be corrupt or open by some other process."
      
    EndIf 
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    Debug "Could not open or create raw database file."                                           ; 
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.41 LTS (Windows - x86)
; CursorPosition = 102
; FirstLine = 17
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP