﻿;==============================================================================================================================================================================
; Import the Food Data Exchange XML data
;==============================================================================================================================================================================

Procedure ImportFoodDataExchangeFile_XML(ImportFileName.s, *CurrentNode, CurrentSublevel.i)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected NodeName.s, *ChildNode, AttributeName.s, AttributeValue.s, NodeText.s, DataLinesCountLoop.i
  
  Protected PictureEncodedDataLines.i, PictureEncodedXMLData.s, CurrentXMLDataLine.s , *Ascii
  
  Protected RecipeInstructions.s
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    NodeName.s = GetXMLNodeName(*CurrentNode)
    
    ;----------------------------------------------------------------------------------------------
    ; Debug NodeName.s
    ;----------------------------------------------------------------------------------------------
    
    If NodeName = "fdx"
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s  = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s
              
            Case "Source"
              
              If AttributeValue.s
                
                Recipe\Importedfrom.s   =  AttributeValue.s
                
              Else
                
                Recipe\Importedfrom.s   = "Recipe compilation"
                
              EndIf
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "Recipe"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Len(Recipe\Recipetitle)
        
        SaveImportRecipe(ImportFileName.s)  ; Save the filled fields of the recipe structure
        
        ClearRecipeStructure()
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s  = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s
              
            Case "Name"
              
              Recipe\Recipetitle.s      = AttributeValue.s
              
              Recipe\Recipetitle.s      = ReplaceString(Recipe\Recipetitle.s, "'", "''")
              
              Recipe\Recipetitle.s      = ProperCaseString(Recipe\Recipetitle.s)              
              
            Case "Servings"
              
              Recipe\Numberofservings.s = AttributeValue.s
              
            Case "RecipeTypes"
              
              Recipe\Categories         = RemoveString(AttributeValue.s, " ")
              
            Case "Source"
              
              Recipe\Recipesource       = Trim(AttributeValue.s)
              
            Case "Comments"
              
              Recipe\Authorcomments.s   = AttributeValue.s
              
            Case "CookingTime"
              
              Recipe\Cookingtime.s      = AttributeValue.s
              
            Case "PreparationTime"
              
              Recipe\Preparationtime.s  = AttributeValue.s
              
            Case "Copyright"
              
              Recipe\Copyright.s        = AttributeValue.s
              
            Case "CreateDate"
              
              Recipe\Updated.s          = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Gather all the ingredient data here
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeIngredient"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      AddElement(Recipe\IngredientList())
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s  = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s
              
            Case "Quantity"
              
              Recipe\IngredientList()\Unit        = AttributeValue.s
              
            Case "Unit"
              
              Recipe\IngredientList()\Measure     = CorrectIngredientMeasure(AttributeValue.s)  ; Corrent single letetr ingredient measure strings
              
            Case "Ingredient"
              
              Recipe\IngredientList()\Ingredient  = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Grab the recipe instructions
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "ProcedureText"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      Recipe\Instructions.s + ReplaceString(RecipeCRLFSpaceTrim(XmlToText(GetXMLNodeText(*CurrentNode))), "'", "''") + #LF$
      
      ;Recipe\Instructions.s = ReplaceString(Recipe\Instructions.s, "'", "''")

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf  NodeName.s = "RecipeReview"
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s  = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName.s
              
            Case "Rating"
              
              Recipe\Reciperating.s = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf      
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeTip"   ; Doesn't seem to get the tips in every recipe if it is a single tip
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      NodeText.s              = ReplaceString(GetXMLNodeText(*CurrentNode), "'", "''")
      
      Recipe\Othercomments.s  + RecipeCRLFSpaceTrim(XmlToText(NodeText.s)) + #LF$ + #LF$
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "RecipeNutrition"
      
      If ExamineXMLAttributes(*CurrentNode)
        
        While NextXMLAttribute(*CurrentNode)
          
          AttributeName.s  = XMLAttributeName(*CurrentNode)
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Recipe\Nutritionaldata.s  + AttributeName.s + " = " + AttributeValue.s  + #LF$
          
        Wend
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Check for recipe images in the XML
      ;--------------------------------------------------------------------------------------------
      
    ElseIf NodeName.s = "SourceImage" Or NodeName.s = "RecipeImage"    ; Image description node
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If ExamineXMLAttributes(*CurrentNode)                            ;
        
        AddElement(Recipe\PictureList())
        
        Recipe\PictureList()\PictureBuffer = AllocateMemory(#PictureBufferSize)
        
        If Recipe\PictureList()\PictureBuffer = #Null
          
          Debug "memory allocation error!"
          
        EndIf
        
        While NextXMLAttribute(*CurrentNode)                            ; 
          
          AttributeName.s = XMLAttributeName(*CurrentNode)              ; 
          
          AttributeValue.s = ReplaceString(XMLAttributeValue(*CurrentNode), "'", "''")
          
          Select AttributeName
              
            Case "FileType"
              
              Recipe\PictureList()\FileType         = AttributeValue.s
              
            Case "FileName"
              
              Recipe\PictureList()\FileName         = AttributeValue.s
              
            Case "Description"
              
              Recipe\PictureList()\Description      = AttributeValue.s
              
            Case "OriginalFileName"
              
              Recipe\PictureList()\OriginalFileName = AttributeValue.s
              
          EndSelect
          
        Wend
        
      EndIf
      
      If Len(Recipe\PictureList()\OriginalFileName) = 0                 ; If the filename came back empty
        
        If Len(Recipe\PictureList()\FileName) = 0
          
          Recipe\PictureList()\FileName = "Dummy." + Recipe\PictureList()\FileType
          
        EndIf
        
        Recipe\PictureList()\OriginalFileName = Recipe\Recipetitle + "." + GetExtensionPart(Recipe\PictureList()\FileName)    ; Add the recipe title as the filename
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Clean up the grabbed XML data so that we can assemble the picture, line by line
      ;--------------------------------------------------------------------------------------------
      
      PictureEncodedXMLData.s    = GetXMLNodeText(*CurrentNode)
      
      PictureEncodedXMLData.s    = RemoveString(PictureEncodedXMLData.s, #LF$)
      
      PictureEncodedXMLData.s    = Trim(PictureEncodedXMLData.s, #CR$)
      
      ; Find out how many lines the data is strung across
      
      PictureEncodedDataLines.i  = CountString(PictureEncodedXMLData.s, #CR$)
      
      ; Proceed if we got data to process with CR's in it
      
      If PictureEncodedDataLines.i
        
        For DataLinesCountLoop.i = 1 To PictureEncodedDataLines.i
          
          CurrentXMLDataLine.s = StringField(PictureEncodedXMLData.s, DataLinesCountLoop.i, #CR$)
          
          If Len(CurrentXMLDataLine.s)
            
            *Ascii = AllocateMemory(StringByteLength(CurrentXMLDataLine.s, #PB_Ascii), #PB_Memory_NoClear)
            
            If *Ascii
              
              PokeS(*Ascii, CurrentXMLDataLine.s, -1, #PB_Ascii|#PB_String_NoZero)
              
              Recipe\PictureList()\PictureBufferPtr + Base64Decoder(@PictureEncodedXMLData, Len(CurrentXMLDataLine.s), Recipe\PictureList()\PictureBuffer + Recipe\PictureList()\PictureBufferPtr, #PictureBufferSize - Recipe\PictureList()\PictureBufferPtr)
              
              FreeMemory(*Ascii)
              
            EndIf
            
          EndIf
          
        Next DataLinesCountLoop.i
        
      Else  ; only one line of data
        
        If Len(PictureEncodedXMLData.s)
          
          *Ascii = AllocateMemory(StringByteLength(PictureEncodedXMLData.s, #PB_Ascii), #PB_Memory_NoClear)
          
          If *Ascii
            
            PokeS(*Ascii, PictureEncodedXMLData.s, -1, #PB_Ascii|#PB_String_NoZero)
            
            Recipe\PictureList()\PictureBufferPtr = Base64Decoder(*Ascii, MemorySize(*Ascii), Recipe\PictureList()\PictureBuffer, #PictureBufferSize)
            
            FreeMemory(*Ascii)
            
          EndIf
          
        EndIf
        
      EndIf
      
      PictureEncodedXMLData.s = #EmptyString
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    While *ChildNode <> 0
      
      ImportFoodDataExchangeFile_XML(ImportFileName, *ChildNode, CurrentSublevel + 1)
      
      *ChildNode = NextXMLNode(*ChildNode)
      
    Wend

    ; to save the last one and free the picture memory
    
    If *ChildNode = 0 And CurrentSublevel = 0
      
      If Len(Recipe\Recipetitle)
        
        SaveImportRecipe(ImportFileName.s)  ; Save the filled fields of the recipe structure
        
        ClearRecipeStructure()
        
      EndIf
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 100
; FirstLine = 373
; Folding = -
; EnableAsm
; EnableUnicode
; EnableXP
; CompileSourceDirectory