﻿;========================================================================================================================================================== 
; Temporary Ingredient list data structure
;==========================================================================================================================================================

Structure IngredientListStructure
  Column.i
  Measure.s
  Unit.s
  Ingredient.s
EndStructure

;================================================================================================================================================================================
; Format the ingredient string for mealmaster recipes
;================================================================================================================================================================================

Procedure ImportMealMasterIngredient(Column.i, Line.s, List IngredientList.IngredientListStructure())
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  AddElement(IngredientList())
  
  IngredientList()\Column       = Column
  
  IngredientList()\Measure      = Trim(Left(Line.s, 7))
  
  IngredientList()\Measure      = CorrectIngredientMeasure(IngredientList()\Measure)  ; Corrent single letetr ingredient measure strings
  
  IngredientList()\Unit         = Trim(Mid(Line.s,  8, 3))
  
  IngredientList()\Ingredient   = Trim(Mid(Line.s, 12))
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

;==============================================================================================================================================================================
; Import a MealMaster recipe dump into RecipeMuncher
;==============================================================================================================================================================================

Procedure ImportMealMasterFile(ImportFileName.s)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  Protected.i Result, State, Pos, Count, i, LastLineWasEmpty, FirstIngredientFound, DoubleColumn
  
  Protected.i RecipeFileHandle, Comment
  
  Protected.s Line, Help, Char, Column
  
  Protected   NewList IngredientList.IngredientListStructure()
  
  ;------------------------------------------------------------------------------------------------
  ;
  ;------------------------------------------------------------------------------------------------
  
  RecipeFileHandle.i = ReadFile(#PB_Any,  ImportFilename.s)
  
  ;------------------------------------------------------------------------------------------------
  ;
  ;------------------------------------------------------------------------------------------------
  
  If RecipeFileHandle.i
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
    While Not Eof(RecipeFileHandle.i)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      Line.s = ReadString(RecipeFileHandle.i, #PB_UTF8)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      Select State

        ;------------------------------------------------------------------------------------------
        ; No recipe found yet
        ;------------------------------------------------------------------------------------------
          
        Case 0  ; Debug "State 0"
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          Pos = FindString(Line.s, "Meal-Master")
        
          If Pos
            
            State + 1
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; Recipe version number found
          ;----------------------------------------------------------------------------------------

        Case 1  ; Debug "State 1"
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          Pos = FindString(Line.s, "Title:")
          
          If Pos
            
            ClearList(IngredientList())
            
            ClearRecipeStructure()
            
            Recipe\Recipetitle.s  = Trim(Mid(Line.s, Pos + 7))
            
            Recipe\Recipetitle.s  = ReplaceString(Recipe\Recipetitle, "'", "''")
            
            Recipe\Recipetitle.s  = ProperCaseString(Recipe\Recipetitle.s)               
            
            State + 1
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; Recipe Categories found
          ;----------------------------------------------------------------------------------------
          
        Case 2  ; Debug "State 2"

          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          Pos = FindString(Line.s, "Categories:")
          
          If Pos
            
            Recipe\Subcategories = ReplaceString(RemoveString(Mid(Line.s, Pos + 12), " "), "'", "''")
            
            State + 1
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; Recipe yield found
          ;----------------------------------------------------------------------------------------
          
        Case 3  ; Debug "State 3"
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          Count = Len(Line.s)
          
          For i = 1 To Count
            
            If Val(Mid(Line.s, i)) > 0
              
              Recipe\Numberofservings = Str(Val(Mid(Line.s, i)))
              
              State + 1
              
              Break
              
            EndIf
            
          Next i
          
          ;----------------------------------------------------------------------------------------
          ; Recipe Ingredients found
          ;----------------------------------------------------------------------------------------
          
        Case 4  ; Debug "State 4"
          
          If Len(Trim(Line)) > 0
            
            If Len(Line.s) > 40
              
              DoubleColumn = #True
              
            EndIf
            
            FirstIngredientFound = #True
            
            If DoubleColumn
              
              Column.s = Mid(Line.s, 42)
              
              ImportMealMasterIngredient(2, Column, IngredientList())
              
            EndIf
            
            Column = Left(Line.s, 41)
            
            ImportMealMasterIngredient(1, Column, IngredientList())
            
            LastLineWasEmpty = #False
            
            State + 1
            
          EndIf
          
        Case 5  ; Debug "State 5" ; Debug Line
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
         
          Pos = FindString(Line.s, "Meal-Master")
          
          If Pos
            
            State = 1
            
          Else
            
            If Len(Trim(Line.s))
              
              If FindString(Line.s, "-----")
                
                LastLineWasEmpty = #False
                
                Line.s = RemoveString(Line.s, "MMMMM")
                
                Line.s = Trim(Line.s)
                
                AddElement(IngredientList())
                
                IngredientList()\Ingredient = Trim(Line.s, "-")
                
                IngredientList()\Column = 1

              Else
                
                Comment = #False
                
                If LastLineWasEmpty
                  
                  Char.s = Trim(Left(Line.s, 3))
                  
                  If Char.s <> ""
                    
                    If Char.s <> ":" ;And Asc(Char.s) > '9'
                      
                      Recipe\Instructions = ReplaceString(Trim(Line.s), "'", "''")
                      
                      State + 1
                      
                    EndIf
                    
                  Else
                    
                    If Trim(Left(Line.s, 7)) = ""
                      
                      AddElement(IngredientList())
                      
                      IngredientList()\Ingredient = Trim(Line.s)
                      
                      IngredientList()\Column = 1
                      
                      Comment = #True
                      
                    EndIf
                    
                  EndIf
                  
                EndIf
                
                If Not Comment And State = 5
                  
                  If DoubleColumn
                    
                    Column = Mid(Line.s, 42)
                    
                    ImportMealMasterIngredient(2, Column, IngredientList())
                    
                  EndIf
                  
                  Column = Left(Line.s, 41)

                  If Left(LTrim(Column), 1) <> "-"
                    
                    ImportMealMasterIngredient(1, Column, IngredientList())
                    
                  Else
                    
                    IngredientList()\Ingredient + " " + LTrim(Trim(Column), "-")
                    
                  EndIf
                  
                EndIf
                
              EndIf
              
              LastLineWasEmpty = #False
              
            Else
              
              If LastLineWasEmpty
                
                State + 1
                
              Else
                
                LastLineWasEmpty = #True
                
              EndIf
              
            EndIf
            
          EndIf
            
          ;----------------------------------------------------------------------------------------
          ; Recipe Instructions found
          ;----------------------------------------------------------------------------------------
          
        Case 6  ; Debug "State 6"
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          Pos = FindString(Line.s, "Meal-Master")
          
          If Pos
            
            State = 1
            
          Else
            
            If Line.s = "MMMMM" Or Line.s = "-----"
              
              Recipe\Subcategories = ReplaceString(Trim(Recipe\Subcategories, ","), "'", "''")
              
              ForEach IngredientList()

                If Len(IngredientList()\Ingredient)
                  
                  If IngredientList()\Column = 1
                    
                    AddElement(Recipe\IngredientList())
                    
                    Recipe\IngredientList()\Measure     = IngredientList()\Measure
                    
                    Recipe\IngredientList()\Unit        = IngredientList()\Unit
                    
                    Recipe\IngredientList()\Ingredient  = IngredientList()\Ingredient
                    
                  EndIf
                  
                EndIf
                
              Next
              
              If DoubleColumn
                
                ForEach IngredientList()

                  If Len(IngredientList()\Ingredient)
                    
                    If IngredientList()\Column = 2
                      
                      AddElement(Recipe\IngredientList())
                      
                      Recipe\IngredientList()\Measure     = IngredientList()\Measure
                      
                      Recipe\IngredientList()\Unit        = IngredientList()\Unit
                      
                      Recipe\IngredientList()\Ingredient  = IngredientList()\Ingredient
                      
                    EndIf
                    
                  EndIf
                  
                Next
                
              EndIf
              
              SaveImportRecipe("MealMaster")
              
              ClearRecipeStructure()
              
              State = 0
              
            Else
            
              If Len(Line.s)

                Line = RecipeCRLFSpaceTrim(Line.s)
                
                If Len(Line.s)
                  
                  Recipe\Instructions + ReplaceString(Line, "'", "''") + #LF$
                  
                EndIf
                
              EndIf
              
            EndIf
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
      EndSelect
      
    Wend
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFile(RecipeFileHandle.i)
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    Debug "File wasn't opened, no there is file handle."
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 343
; FirstLine = 340
; Folding = -
; EnableXP
; CompileSourceDirectory